@@ -1,83 +1,61 @@
 /* dynl.c - dynamic linking
- *
- * Copyright (C) 1990, 91, 92, 93, 94, 95, 96, 97, 98, 99, 2000, 2001, 2002,
- * 2003, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+
+   Copyright 1990-2003,2008-2011,2017-2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
+/* "dynl.c" dynamically link&load object files.
+   Author: Aubrey Jaffer
+   Modified for libguile by Marius Vollmer */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
 #include <alloca.h>
-#include <string.h>
-
-/* "dynl.c" dynamically link&load object files.
-   Author: Aubrey Jaffer
-   Modified for libguile by Marius Vollmer */
-
-#if 0 /* Disabled until we know for sure that it isn't needed */
-/* XXX - This is only here to drag in a definition of __eprintf. This
-   is needed for proper operation of dynamic linking. The real
-   solution would probably be a shared libgcc. */
-
-#undef NDEBUG
-#include <assert.h>
-
-static void
-maybe_drag_in_eprintf ()
-{
-  assert (!maybe_drag_in_eprintf);
-}
-#endif
-
-#include <stdlib.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
-#include "libguile/_scm.h"
-#include "libguile/libpath.h"
-#include "libguile/dynl.h"
-#include "libguile/smob.h"
-#include "libguile/keywords.h"
-#include "libguile/ports.h"
-#include "libguile/strings.h"
-#include "libguile/deprecation.h"
-#include "libguile/validate.h"
-#include "libguile/dynwind.h"
-#include "libguile/foreign.h"
-#include "libguile/gc.h"
-
 #include <ltdl.h>
 
-/*
-  From the libtool manual: "Note that libltdl is not threadsafe,
-  i.e. a multithreaded application has to use a mutex for libltdl.".
+#include "deprecation.h"
+#include "dynwind.h"
+#include "foreign.h"
+#include "gc.h"
+#include "gsubr.h"
+#include "keywords.h"
+#include "libpath.h"
+#include "list.h"
+#include "ports.h"
+#include "smob.h"
+#include "strings.h"
+#include "threads.h"
 
-  Guile does not currently support pre-emptive threads, so there is no
-  mutex.  Previously SCM_CRITICAL_SECTION_START and
-  SCM_CRITICAL_SECTION_END were used: they are mentioned here in case
-  somebody is grepping for thread problems ;)
-*/
-/* njrev: not threadsafe, protection needed as described above */
+#include "dynl.h"
 
 
+/* From the libtool manual: "Note that libltdl is not threadsafe,
+   i.e. a multithreaded application has to use a mutex for libltdl.".
+   Note: We initialize it as a recursive mutex below.  */
+static scm_i_pthread_mutex_t ltdl_lock = SCM_I_PTHREAD_MUTEX_INITIALIZER;
+
 /* LT_PATH_SEP-separated extension library search path, searched last */
 static char *system_extensions_path;
 
@@ -259,6 +237,7 @@ SCM_DEFINE (scm_dynamic_link, "dynamic-link", 0, 1, 0,
   char *file;
 
   scm_dynwind_begin (0);
+  scm_i_dynwind_pthread_mutex_lock (&ltdl_lock);
 
   if (SCM_UNBNDP (filename))
     file = NULL;
@@ -301,13 +280,18 @@ SCM_DEFINE (scm_dynamic_unlink, "dynamic-unlink", 1, 0, 0,
 {
   /*fixme* GC-problem */
   SCM_VALIDATE_SMOB (SCM_ARG1, dobj, dynamic_obj);
+
+  scm_dynwind_begin (0);
+  scm_i_dynwind_pthread_mutex_lock (&ltdl_lock);
   if (DYNL_HANDLE (dobj) == NULL) {
     SCM_MISC_ERROR ("Already unlinked: ~S", scm_list_1 (dobj));
   } else {
     sysdep_dynl_unlink (DYNL_HANDLE (dobj), FUNC_NAME);
     SET_DYNL_HANDLE (dobj, NULL);
-    return SCM_UNSPECIFIED;
   }
+  scm_dynwind_end ();
+
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -335,6 +319,7 @@ SCM_DEFINE (scm_dynamic_pointer, "dynamic-pointer", 2, 0, 0,
       char *chars;
 
       scm_dynwind_begin (0);
+      scm_i_dynwind_pthread_mutex_lock (&ltdl_lock);
       chars = scm_to_locale_string (name);
       scm_dynwind_free (chars);
       val = sysdep_dynl_value (chars, DYNL_HANDLE (dobj), FUNC_NAME);
@@ -400,12 +385,13 @@ scm_init_dynamic_linking ()
 {
   scm_tc16_dynamic_obj = scm_make_smob_type ("dynamic-object", 0);
   scm_set_smob_print (scm_tc16_dynamic_obj, dynl_obj_print);
-  sysdep_dynl_init ();
-#include "libguile/dynl.x"
-}
 
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
+  /* Make LTDL_LOCK recursive so that a pre-unwind handler can still use
+     'dynamic-link', as is the case at the REPL.  See
+     <https://bugs.gnu.org/29275>.  */
+  scm_i_pthread_mutex_init (&ltdl_lock,
+			    scm_i_pthread_mutexattr_recursive);
+
+  sysdep_dynl_init ();
+#include "dynl.x"
+}
